class Context:
  def __init__(self, *args):
    self.providers = args

  def _render_fields(self):
    fields = ""
    for provider in self.providers:
      for field in provider.fields:
        fields += f"({provider.name}) {field.name} -> {field.type}\n"
    return fields
  
  def __repr__(self):
    return f"=== [ Providers ] ===\n{', '.join([i.name for i in self.providers])}\n=== [ Fields ] ===\n{self._render_fields()}"


from enum import Enum

class Frequency(Enum):
  TICK = 'tick',
  SECOND = '1s',
  MINUTE = '1m',
  HOUR = '1hr',
  DAY = '1d',
  WEEK = '1wk',
  MONTH = '1mo',
  QUARTER = '1q',
  YEAR = '1y'

class Strategy:
  """
  Strategies contains the logic for evaluating context and creating orders ONLY. Should always remain ambiguous to the broker and target asset

  This class in particular templates the methods of a strategy. 

  """

  def __init__(self, 
    frequency: Frequency = Frequency.DAY):
    frequency: Frequency = frequency

  def on_tick(self, tick):
    pass

  def on_start(self):
    pass

  def on_end(self):
    pass

  def on_market_open():
    pass

  def on_market_close():
    pass



import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

setup(
  name='terrace',
  version='1.0.0',
  url='https://github.com/carterjfulcher/terrace',
  author='carterjfulcher',
  author_email='carter@fulcheranalytics.io',
  packages=find_packages(exclude=("tests",)),
  platforms='any',
  license='MIT',
  ext_modules=[],
  description="an open source algorithmic trading engine optimized for a great developer experience",
  long_description='See https://github.com/carterjfulcher/terrace',
)


from terrace.data.providers import Provider
from terrace.data import Field 
import pandas as pd 
from pydantic import validate_arguments
from pydantic.fields import Optional
from typing import List

class FileProvider(Provider):
  @validate_arguments
  def __init__(self, filename: str, index: Optional[str] = None, columns: Optional[List[str]] = None, name: Optional[str] = "CSV"):
    self.filename = filename
    self.index = index
    self.name = name 
    self.columns = columns
    super().__init__()

  def _load(self, df: pd.DataFrame):
    if self.columns == None:
      self.columns = df.columns
    for col in [i for i in [*self.columns, self.index] if i is not None]:
      if col not in df.columns:
        raise ValueError(f"Column {col} not found in {self.filename}")
    if self.index is not None:
      self.df.set_index(self.index, inplace=True)
    if self.columns is not None:
      self.df = df[self.columns]
    else:
      self.df = df
    self._load_fields_from_df()

  def _load_csv(self):
    self._load(pd.read_csv(self.filename))

  def _load_excel(self):
    self._load(pd.read_excel(self.filename))

  def _load_fields_from_df(self):
    for col in self.df.columns:
      self.fields.append(Field(name=col, type=self.df[col].dtype.name, description="", provider=self.name))

class CSV(FileProvider):
  def __init__(self, filename: str, index: Optional[str] = None, columns: Optional[List[str]] = None):
    super().__init__(filename, index, columns)
    self._load_csv()

class Excel(FileProvider):
  def __init__(self, filename: str, index: Optional[str] = None, columns: Optional[List[str]] = None):
    super().__init__(filename, index, columns)
    self._load_excel()
